@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.17 2002/04/19 12:07:33 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.22 2005/05/11 12:50:46 abe Exp $";
 #endif
 
 
@@ -80,10 +80,26 @@ struct rnode {
 
 #    if	AIXV>4330
 struct rnode64 {
+
+#     if AIXV<5200
 	caddr_t	r_d1[11];		/* dummies; links? */
+#     else /* AIXV>=5200 */
+#      if	AIXV<5300
+	caddr_t	r_d1[12];		/* dummies; links? */
+#      else	/* AIXV>=5300 */
+	caddr_t r_d1[7];		/* dummies; links? */
+#      endif	/* AIXV<5300 */
+#     endif /* AIXV<5200 */
+
 	struct vnode r_vnode;		/* vnode for remote file */
 	struct gnode r_gnode;		/* gnode for remote file */
+
+#     if	AIXV<5300
 	caddr_t r_d2[15];		/* dummies; rnode elements? */
+#     else	/* AIXV>=5300 */
+	caddr_t r_d2[11];		/* dummies; rnode elements? */
+#     endif	/* AIXV<5300 */
+
 	off_t r_size;			/* client's view of file size (long)*/
 	struct vattr r_attr;		/* cached vnode attributes */
 };
@@ -173,7 +189,7 @@ process_node(va)
 	dev_t dev, rdev;
 	int devs = 0;
 	struct gnode g;
-	struct inode i;
+	struct l_ino i;
 	int ic = 0;
 	int ins = 0;
 	struct vfs *la = NULL;
@@ -216,6 +232,21 @@ process_node(va)
 # endif	/* AIXA<2 */
 #endif	/* defined(HAS_NFS) */
 
+#if	defined(HAS_SANFS)
+	struct sanfs_node {	/* DEBUG */
+
+	/*
+	 * This is a DEBUG version of the SANFS node structure.  When IBM makes
+	 * the SANFS header files available in /usr/include, this definition
+	 * will be removed.
+	 */
+	    u_long san_d1[20];	/* DEBUG */
+	    struct gnode san_gnode;	/* DEBUG */
+	    u_long san_d2[128];	/* DEBUG */
+	} san;
+	int sans = 0;
+#endif	/* defined(HAS_SANFS) */
+
 #if	AIXV>=4140
 	struct clone *cl;
 	KA_T ka;
@@ -344,7 +375,7 @@ process_node(va)
 		    enter_nm(Namech);
 		    return;
 		}
-		if (!g.gn_data || readinode((KA_T)g.gn_data, &i)) {
+		if (!g.gn_data || readlino(&g, &i)) {
 		    (void) snpf(Namech, Namechl,
 			"pfsgnode at %s: can't read inode (%s)",
 			print_kptr((KA_T)sn.sn_pfsgnode, tbuf, sizeof(tbuf)),
@@ -531,28 +562,46 @@ process_clone_again:
 	    }
 	}
 /*
- * Set special node types: NFS; or PROC.
+ * Set special node types: NFS, PROC or SANFS.
  */
 
-#if	defined(HAS_NFS)
+#if	defined(MNT_REMOTE)
 	if (vfs && vfs->vmt_flags & MNT_REMOTE) {
+	    switch(vfs->vmt_gfstype) {
 
-# if	defined(HAS_AFS)
-	    if (!AFSVfsp || (KA_T)v->v_vfsp != AFSVfsp)
-# endif	/* defined(HAS_AFS) */
+# if	defined(HAS_NFS)
+	    case MNT_NFS:
 
-		Ntype = N_NFS;
+#  if	defined(MNT_NFS3)
+	    case MNT_NFS3:
+#  endif	/* defined(MNT_NFS3) */
+
+#  if	defined(MNT_NFS4)
+	    case MNT_NFS4:
+#  endif	/* defined(MNT_NFS4) */
+
+#  if	defined(HAS_AFS)
+		if (!AFSVfsp || (KA_T)v->v_vfsp != AFSVfsp)
+#  endif	/* defined(HAS_AFS) && defined(HAS_NFS) */
+
+		    Ntype = N_NFS;
+# endif	/* defined(HAS_NFS) */
+		break;
+
+# if	defined(HAS_SANFS) && defined(MNT_SANFS)
+	    case MNT_SANFS:
+		Ntype = N_SANFS;
+		break;
+# endif	/* defined(HAS_SANFS) && defined(MNT_SANFS) */
+
+	    }
 	}
-# if	defined(HASPROCFS)
-	else if (vfs && (vfs->vmt_gfstype == MNT_PROCFS))
-	    Ntype = N_PROC;
-# endif	/* defined(HASPROCFS) */
-#else	/* !defined(HAS_NFS) */
-# if	defined(HASPROCFS)
+#endif	/* defined(MNT_REMOTE) */
+
+#if	defined(HASPROCFS)
 	if (vfs && (vfs->vmt_gfstype == MNT_PROCFS))
 	    Ntype = N_PROC;
-# endif	/* defined(HASPROCFS) */
-#endif	/* defined(HAS_NFS) */
+#endif	/* defined(HASPROCFS) */
 
 /*
  * Get the lock status.
@@ -577,18 +626,16 @@ process_clone_again:
 #  if	AIXV<=4330
 		width = 32;
 #  else	/* AIXV>4330 */
-		switch ((width = _system_configuration.width)) {
-		case 32:
-		    break;
-		case 64:
-		   rp = (char *)&r64;
-		   rsz = sizeof(r64);
-		   break;
-		default:
+		if (__KERNEL_64()) {
+		    width = 64;
+		    rp = (char *)&r64;
+		    rsz = sizeof(r64);
+		} else if (__KERNEL_32()) {
+		    width = 32;
+		} else {
 		    if (!Fwarn)
 			(void) fprintf(stderr,
-			    "Pn: WARNING: unknown architecture bit size: %d\n",
-			    Pn, width);
+			    "%s: WARNING: unknown kernel bit size\n", Pn);
 		    width = -2;
 		}
 #  endif	/* AIXV<-4330 */
@@ -625,6 +672,29 @@ process_clone_again:
 	    break;
 #endif	/* defined(HAS_NFS) */
 
+#if	defined(HAS_SANFS)
+/*
+ * Read SANFS node and associated structures.
+ */
+	case N_SANFS:
+	    if (!g.gn_data
+	    ||  kread((KA_T)g.gn_data, &san, sizeof(san))
+	    ) {
+		(void) snpf(Namech, Namechl, "gnode at %s has no SANFS node",
+		    print_kptr((KA_T)v->v_gnode, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	/*
+	 * DEBUG: this code is insufficient.  It can't be completed until IBM
+	 * makes the SANFS header files available in /usr/include.  There are
+	 * apparently two node structures following the SANFS node and file
+	 * attributes (size, etc.) are in the second structure.
+	 */
+	    sans = 1;
+	    break;
+#endif	/* defined(HAS_SANFS) */
+
 /*
  * Read N_REGLR nodes.
  */
@@ -641,14 +711,15 @@ process_clone_again:
 		    return;
 		}
 		(void) zeromem((char *)&i, sizeof(i));
-		i.i_number = c.cn_inumber;
-		i.i_size = (off_t)c.cn_size;
+		i.number = c.cn_inumber;
+		i.size = (off_t)c.cn_size;
+		i.number_def = i.size_def = 1;
 	    /*
 	     * Otherwise, read the inode.
 	     */
 
 	    } else if (g.gn_data) {
-		if (readinode((KA_T)g.gn_data, &i)) {
+		if (readlino(&g, &i)) {
 		    (void) snpf(Namech, Namechl,
 			"gnode at %s can't read inode: %s",
 			print_kptr((KA_T)v->v_gnode, tbuf, sizeof(tbuf)),
@@ -740,6 +811,14 @@ process_clone_again:
 	else
 #endif	/* defined(HASPROCFS) */
 
+#if	defined(HAS_SANFS)
+	if ((Ntype == N_SANFS) && vfs) {
+	    dev = vfs->dev;
+	    devs = 1;
+	}
+	else
+#endif	/* defined(HAS_SANFS) */
+
 	{
 	    if (vfs) {
 		dev = vfs->dev;
@@ -779,6 +858,20 @@ process_clone_again:
 	    break;
 #endif	/* defined(HAS_NFS) */
 
+#if	defined(HAS_SANFS)
+	case N_SANFS:
+	    if (sans) {
+	    
+	    /*
+	     * DEBUG: this code is insufficient.  It can't be completed until
+	     * IBM makes the SANFS header files available in /usr/include.
+	     */
+		/* Lf->inode = ???	DEBUG */
+		Lf->inp_ty = 1;
+	    }
+	    break;
+#endif	/* defined(HAS_SANFS) */
+
 # if	AIXV>=3200
 	case N_BLK:
 	case N_CHR:
@@ -788,8 +881,8 @@ process_clone_again:
 
 	case N_REGLR:
 	    if (ins) {
-		Lf->inode = (unsigned long) i.i_number;
-		Lf->inp_ty = 1;
+		Lf->inode = (unsigned long) i.number;
+		Lf->inp_ty = i.number_def;
 	    }
 	}
 /*
@@ -823,6 +916,21 @@ process_clone_again:
 		break;
 #endif	/* defined(HAS_NFS) */
 
+#if	defined(HAS_SANFS)
+	    case N_SANFS:
+		if (sans) {
+
+		/*
+	 	 * DEBUG: this code is insufficient.  It can't be completed
+		 * until IBM makes the SANFS header files available in
+		 * /usr/include.
+		 */
+		    /* Lf->sz = (SZOFFTYPE)???	DEBUG */
+		    Lf->sz_def = 1;
+		}
+		break;
+#endif	/* defined(HAS_SANFS) */
+
 #if	 AIXV>=3200
 	    case N_BLK:
 		if (!Fsize)
@@ -838,8 +946,8 @@ process_clone_again:
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
 		    if (ins) {
-			Lf->sz = (SZOFFTYPE)i.i_size;
-			Lf->sz_def = 1;
+			Lf->sz = (SZOFFTYPE)i.size;
+			Lf->sz_def = i.size_def;
 		    }
 		} else if (((type == VBLK) || (type == VCHR) || (type == VMPC))
 		       &&  !Fsize)
@@ -869,6 +977,21 @@ process_clone_again:
 		break;
 #endif	/* defined(HAS_NFS) */
 
+#if	defined(HAS_SANFS)
+	    case N_SANFS:
+		if (sans) {
+
+		/*
+	 	 * DEBUG: this code is insufficient.  It can't be completed
+		 * until IBM makes the SANFS header files available in
+		 * /usr/include.
+		 */
+		    /* Lf->nlink = (long)???	DEBUG */
+		    Lf->nlink_def = 1;
+		}
+		break;
+#endif	/* defined(HAS_SANFS) */
+
 #if	AIXV>=3200
 	    case N_BLK:
 	    case N_CHR:
@@ -878,8 +1001,8 @@ process_clone_again:
 
 	    case N_REGLR:
 		if (ins) {
-		    Lf->nlink = (long)i.i_nlink;
-		    Lf->nlink_def = 1;
+		    Lf->nlink = (long)i.nlink;
+		    Lf->nlink_def = i.nlink_def;
 		}
 		break;
 	    }
@@ -1103,3 +1226,65 @@ process_shmt(sa)
 	enter_nm(Namech);
 }
 #endif	/* AIXV>=4200 */
+
+
+/*
+ * readlino() -- read local inode
+ */
+
+int
+readlino(ga, li)
+	struct gnode *ga;			/* gnode address */
+	struct l_ino *li;			/* local inode receiver */
+{
+	struct inode i;				/* "regular" inode */
+
+#if	defined(HAS_JFS2)
+	static struct vnodeops *j2va = (struct vnodeops *)NULL;
+						/* j2_vnops address */
+	static int j2vas = 0;			/* j2nl[] status */
+#endif	/* defined(HAS_JFS2) */
+
+	zeromem((char *)li, sizeof(struct l_ino));
+	if (!ga || !ga->gn_data)
+	    return(0);
+
+#if	defined(HAS_JFS2)
+	if (!j2vas) {
+
+	/*
+	 * Get the j2_vnops address once.
+	 */
+	    struct nlist j2nl[] = {
+		{ "j2_vnops"	},
+		{ (char *)NULL	}
+	    };
+
+	    if (nlist(N_UNIX, j2nl) == 0)
+		j2va = (struct vnodeops *)j2nl[0].n_value;
+	    if (!j2va && !Fwarn) {
+		(void) fprintf(stderr,
+		    "%s: WARNING: can't identify jfs2 files\n", Pn);
+	    }
+	    j2vas = 1;
+	}
+/*
+ * If this system has jfs2, see if this gnode's operation structure pointer
+ * references j2_vnops.
+ */
+	if (ga->gn_ops && j2va && (ga->gn_ops == j2va))
+	    return(readj2lino(ga, li));
+#endif	/* defined(HAS_JFS2) */
+
+/*
+ * Read a "standard" inode.
+ */
+	if (readinode((KA_T)ga->gn_data, &i))
+	    return(1);
+	li->dev = i.i_dev;
+	li->nlink = i.i_nlink;
+	li->number = i.i_number;
+	li->size = i.i_size;
+	li->dev_def = li->nlink_def = li->number_def = li->size_def = 1;
+	return(0);
+}
