@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.21 2004/03/10 23:49:13 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.22 2005/05/11 12:50:46 abe Exp $";
 #endif
 
 
@@ -61,7 +61,7 @@ process_socket(sa)
 	struct domain d;
 	unsigned char *fa = (unsigned char *)NULL;
 	int fam;
-	int fp, lp;
+	int fp, lp, uo;
 	struct gnode g;
 	struct l_ino i;
 	struct inpcb inp;
@@ -312,8 +312,16 @@ process_socket(sa)
 		}
 
 #if	AIXV>=3200
-		ua = (struct sockaddr_un *)((char *)&mb
-		   + (mb.m_hdr.mh_data - (caddr_t)unp.unp_addr));
+		uo = (int)(mb.m_hdr.mh_data - (caddr_t)unp.unp_addr);
+		if ((uo + sizeof(struct sockaddr)) <= sizeof(mb))
+		    ua = (struct sockaddr_un *)((char *)&mb + uo);
+		else {
+		    if (mb.m_hdr.mh_data
+		    &&  !kread((KA_T)mb.m_hdr.mh_data, (char *)&un, sizeof(un))
+		    ) {
+			ua = &un;
+		    }
+		}
 #else	/* AIXV<3200 */
 		ua = (struct sockaddr_un *)(((char *)&mb) + mb.m_off);
 #endif	/* AIXV>=3200 */
