@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.33 2002/02/26 15:18:09 abe Exp $
+ * $Id: machine.h,v 1.38 2004/10/17 21:54:02 abe Exp $
  */
 
 
@@ -94,9 +94,17 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
  */
 
-#if	AIXV>=4200
+# if	AIXV>=4200
 #define	CAN_USE_CLNT_CREATE	1
-#endif	/* AIXV>=4200 */
+# endif	/* AIXV>=4200 */
+
+
+/*
+ * DEVDEV_PATH defines the path to the directory that contains device
+ * nodes.
+ */
+
+#define	DEVDEV_PATH	"/dev"
 
 
 /*
@@ -251,6 +259,14 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define SETLFILEADD Lf->... */
 
 
+/*
+ * HASMNTSUP is defined for those dialects that support the mount supplement
+ * option.
+ */
+
+/* #define	HASMNTSUP	1	*/
+
+
 /*
  * HASMOPT is defined for those dialects that support the reading of
  * kernel memory from an alternate file.
@@ -264,16 +280,12 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  * that lsof can search.  A value of 1 directs printname() to prefix the
  * cache value with the file system directory name; 2, avoid the prefix.
  *
- * HASNCAPID is defined for those dialects with a searchable kernel name
- * cache whose cache and vnodes are linked by a capability ID.
- *
  * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
  *
  * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 /* #define	HASNCACHE	1	*/
-/* #define	HASNCAPID	1	*/
 /* #define	NCACHELDPFX	??? */
 /* #define	NCACHELDSFX	??? */
 
@@ -395,15 +407,15 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  * by inode number.
  */
 
-#if	AIXA<1
+# if	AIXA<1
 /* #define	HASPROCFS	"proc?" */
 /* #define	HASFSTYPE	1 */
 /* #define	HASPINODEN	1 */
-#else	/* AIXA>=1 */
+# else	/* AIXA>=1 */
 #define	HASPROCFS	"proc"
 #define	HASFSTYPE	2
 #define	HASPINODEN	1
-#endif	/* AIXA<1 */
+# endif	/* AIXA<1 */
 
 
 /*
@@ -426,13 +438,33 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASSECURITY	1	*/
 
 
+/*
+ * If HASSECURITY is defined, define HASNOSOCKSECURITY to allow users
+ * restricted by HASSECURITY to list any open socket files, provide their
+ * listing is selected by the "-i" option.
+ */
+
+/* #define	HASNOSOCKSECURITY	1	*/
+
+
 /*
  * HASSETLOCALE is defined for those dialects that have <locale.h> and
  * setlocale().
+ *
+ * If the dialect also has wide character support for language locales,
+ * HASWIDECHAR activates lsof's wide character support and WIDECHARINCL
+ * defines the header file (if any) that must be #include'd to use the
+ * mblen() and mbtowc() functions.
  */
 
 #define	HASSETLOCALE	1
 
+# if	AIXV>=4320
+#define	HASWIDECHAR	1
+# endif	/* AIXV>=4320 */
+
+/* #define	WIDECHARINCL	<wchar.h>	*/
+
 
 /*
  * HASSNODE is defined for those dialects that have snodes.
@@ -441,6 +473,17 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASSNODE	1 */
 
 
+/*
+ * HASSOOPT, HASSOSTATE and HASTCPOPT define the availability of information
+ * on socket options (SO_* symbols), socket states (SS_* symbols) and TCP
+ * options.
+ */
+
+#define	HASSOOPT	1	/* has socket option information */
+#define	HASSOSTATE	1	/* has socket state information */
+#define	HASTCPOPT	1	/* has TCP options or flags */
+
+
 /*
  * Define HASSPECDEVD to be the name of a function that handles the results
  * of a successful stat(2) of a file name argument.
