@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.33 2004/12/30 18:41:31 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.34 2005/05/11 12:52:50 abe Exp $";
 #endif
 
 
@@ -62,15 +62,14 @@ getmemsz(pid)
 	struct vmspace vm;
 
 	for (n = 0, p = P; n < Np; n++, p++) {
-		if (p->P_PID == pid) {
-			if (!p->P_VMSPACE
-			||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
-				return;
-			Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
-							     + vm.vm_ssize);
-			Lf->sz_def = 1;
-			return;
-		}
+	    if (p->P_PID == pid) {
+		if (!p->P_VMSPACE
+		||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
+		    return;
+		Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize+vm.vm_dsize+vm.vm_ssize);
+		Lf->sz_def = 1;
+		return;
+	    }
 	}
 }
 # endif	/* defined(HASPROCFS) */
@@ -213,6 +212,13 @@ process_node(va)
 #endif	/* defined(HASFDESCFS) */
 
 #if	FREEBSDV>=5000
+# if	defined(HAS_UFS1_2)
+	int ufst;
+	struct ufsmount um;
+	struct ufs1_dinode d1;
+	struct ufs2_dinode d2;
+# endif	/* !defined(HAS_UFS1_2) */
+
 	struct cdev cd;
 	int cds;
 	struct devfs_dirent de;
@@ -285,8 +291,11 @@ process_overlaid_node:
 #if	FREEBSDV<5000
 	m = (struct mfsnode *)NULL;
 #else	/* FREEBSDV>=5000 */
-	d = (struct devfs_dirent *)NULL;
 	cds = 0;
+	d = (struct devfs_dirent *)NULL;
+# if	defined(HAS_UFS1_2)
+	ufst = 0;
+# endif	/* !defined(HAS_UFS1_2) */
 #endif	/* FREEBSDV<5000 */
 
 #if	defined(HASPROCFS)
@@ -370,7 +379,7 @@ process_overlaid_node:
 
 #if	FREEBSDV>=5000
 /*
- * Get the FreeBSD 5 and above v_rdev structure for VCHR and VBLD vnodes.
+ * For FreeBSD 5 and above VCHR and VBLK vnodes get the v_rdev structure.
  */
 	if (((v->v_type == VCHR) || (v->v_type == VBLK))
 	&&  v->v_rdev
@@ -603,6 +612,20 @@ process_overlaid_node:
 		return;
 	    }
 	    i = &ib;
+
+# if	defined(HAS_UFS1_2)
+	    if (i->i_ump && !kread((KA_T)i->i_ump, (char *)&um, sizeof(um))) {
+		if (um.um_fstype == UFS1) {
+		    if (i->i_din1
+		    &&  !kread((KA_T)i->i_din1, (char *)&d1, sizeof(d1)))
+			ufst = 1;
+		} else {
+		    if (i->i_din2
+		    &&  !kread((KA_T)i->i_din2, (char *)&d2, sizeof(d2)))
+			ufst = 2;
+		}
+	    }
+# endif	/* defined(HAS_UFS1_2) */
 #endif	/* FREEBSDV<2000 */
 
 	    if ((lff = i->i_lockf)) {
@@ -675,7 +698,13 @@ process_overlaid_node:
 #if	FREEBSDV>=4000
 	    if (i->i_dev
 	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))) {
+
+# if	defined(HAS_NO_SI_UDEV)
+		dev = dev2udev(&si);
+# else	/* !defined(HAS_NO_SI_UDEV) */
 		dev = si.si_udev;
+# endif	/* defined(HAS_NO_SI_UDEV) */
+
 		devs = 1;
 	    }
 #else	/* FREEBSDV<4000 */
@@ -686,8 +715,24 @@ process_overlaid_node:
 	    if ((type == VCHR) || (type == VBLK)) {
 
 #if	FREEBSDV>=5000
+# if	defined(HAS_UFS1_2)
+		if (ufst == 1) {
+		    rdev = d1.di_rdev;
+		    rdevs = 1;
+		} else if (ufst == 2) {
+		    rdev = d2.di_rdev;
+		    rdevs = 1;
+		} else
+# endif	/* defined(HAS_UFS1_2) */
+
 		if (cds) {
+
+# if	defined(HAS_NO_SI_UDEV)
+		    rdev = dev2udev(&cd);
+# else	/* !defined(HAS_NO_SI_UDEV) */
 		    rdev = cd.si_udev;
+# endif	/* defined(HAS_NO_SI_UDEV) */
+
 		    rdevs = 1;
 		}
 #else	/* FREEBSDV<5000 */
@@ -742,9 +787,28 @@ process_overlaid_node:
 		dev = DevDev;
 		devs = 1;
 	    }
-	    if ((type == VCHR) && cds) {
-		rdev = cd.si_udev;
-		rdevs = 1;
+	    if ((type == VCHR)) {
+
+# if	defined(HAS_UFS1_2)
+		if (ufst == 1) {
+		    rdev = d1.di_rdev;
+		    rdevs = 1;
+		} else if (ufst == 2) {
+		    rdev = d2.di_rdev;
+		    rdevs = 1;
+		} else
+# endif	/* defined(HAS_UFS1_2) */
+
+		if (cds) {
+
+# if	defined(HAS_NO_SI_UDEV)
+		    rdev = dev2udev(&cd);
+# else	/* !defined(HAS_NO_SI_UDEV) */
+		    rdev = cd.si_udev;
+# endif	/* defined(HAS_NO_SI_UDEV) */
+
+		    rdevs = 1;
+		}
 	    }
 	}
 #endif	/* FREEBSDV>=5000 */
@@ -865,6 +929,15 @@ process_overlaid_node:
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
 		    if (i) {
+
+#if	defined(HAS_UFS1_2)
+			if (ufst == 1)
+			    Lf->sz = (SZOFFTYPE)d1.di_size;
+			else if (ufst == 2)
+			    Lf->sz = (SZOFFTYPE)d2.di_size;
+			else
+#endif	/* defined(HAS_UFS1_2) */
+
 			Lf->sz = (SZOFFTYPE)i->i_size;
 			Lf->sz_def = 1;
 		    }
