@@ -46,6 +46,21 @@ static char *rcsid = "$Id";
 _PROTOTYPE(static void printpsproto,(uint32_t p));
 
 
+/*
+ * Local macroes
+ */
+
+#if	defined(HASIPv6)
+
+/*
+ * IPv6_2_IPv4()  -- macro to define the address of an IPv4 address contained
+ *		     in an IPv6 address
+ */
+
+#define IPv6_2_IPv4(v6)	(((uint8_t *)((struct in6_addr *)v6)->s6_addr)+12)
+#endif	/* defined(HASIPv6) */
+
+
 /*
  * printpsproto() -- print PSTAT protocol name
  */
@@ -262,7 +277,7 @@ print_tcptpi(nl)
 	    ps++;
 	}
 
-# if	defined(HASTCPTPIQ)
+#if	defined(HASTCPTPIQ)
 	if (Ftcptpi & TCPTPI_QUEUES) {
 	    if (Lf->lts.rqs) {
 		if (Ffield)
@@ -293,9 +308,292 @@ print_tcptpi(nl)
 		ps++;
 	    }
 	}
-# endif	/* defined(HASTCPTPIQ) */
+#endif	/* defined(HASTCPTPIQ) */
 
-# if	defined(HASTCPTPIW)
+#if	defined(HASSOOPT)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int opt;
+
+	    if ((opt = Lf->lts.opt) || Lf->lts.qlens || Lf->lts.qlims) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cSO", sep);
+		ps++;
+		sep = '=';
+
+# if	defined(PS_SO_ACCEPTCONN)
+		if (opt & PS_SO_ACCEPTCONN) {
+		    (void) printf("%cACCEPTCONN", sep);
+		    opt &= ~PS_SO_ACCEPTCONN;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_ACCEPTCONN) */
+
+# if	defined(PS_SO_BROADCAST)
+		if (opt & PS_SO_BROADCAST) {
+		    (void) printf("%cBROADCAST", sep);
+		    opt &= ~PS_SO_BROADCAST;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_BROADCAST) */
+
+# if	defined(PS_SO_DEBUG)
+		if (opt & PS_SO_DEBUG) {
+		    (void) printf("%cDEBUG", sep);
+		    opt &= ~PS_SO_DEBUG;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_DEBUG) */
+
+# if	defined(PS_SO_DONTROUTE)
+		if (opt & PS_SO_DONTROUTE) {
+		    (void) printf("%cDONTROUTE", sep);
+		    opt &= ~PS_SO_DONTROUTE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_DONTROUTE) */
+
+# if	defined(PS_SO_GETIFADDR)
+		if (opt & PS_SO_GETIFADDR) {
+		    (void) printf("%cGETIFADDR", sep);
+		    opt &= ~PS_SO_GETIFADDR;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_GETIFADDR) */
+
+# if	defined(PS_SO_INPCB_COPY)
+		if (opt & PS_SO_INPCB_COPY) {
+		    (void) printf("%cINPCB_COPY", sep);
+		    opt &= ~PS_SO_INPCB_COPY;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_INPCB_COPY) */
+
+# if	defined(PS_SO_KEEPALIVE)
+		if (opt & PS_SO_KEEPALIVE) {
+		    (void) printf("%cKEEPALIVE", sep);
+		    if (Lf->lts.kai)
+			(void) printf("=%d", Lf->lts.kai);
+		    opt &= ~PS_SO_KEEPALIVE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_KEEPALIVE) */
+
+# if	defined(PS_SO_LINGER)
+		if (opt & PS_SO_LINGER) {
+		    (void) printf("%cLINGER", sep);
+		    if (Lf->lts.ltm)
+			(void) printf("=%d", Lf->lts.ltm);
+		    opt &= ~PS_SO_LINGER;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_LINGER) */
+
+# if	defined(PS_SO_OOBINLINE)
+		if (opt & PS_SO_OOBINLINE) {
+		    (void) printf("%cOOBINLINE", sep);
+		    opt &= ~PS_SO_OOBINLINE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_OOBINLINE) */
+
+# if	defined(PS_SO_PMTU)
+		if (opt & PS_SO_PMTU) {
+		    (void) printf("%cPMTU", sep);
+		    opt &= ~PS_SO_PMTU;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_PMTU) */
+
+		if (Lf->lts.qlens) {
+		    (void) printf("%cQLEN=%d", sep, Lf->lts.qlen);
+		    sep = ',';
+		}
+		if (Lf->lts.qlims) {
+		    (void) printf("%cQLIM=%lu", sep, Lf->lts.qlim);
+		    sep = ',';
+		}
+
+# if	defined(PS_SO_REUSEADDR)
+		if (opt & PS_SO_REUSEADDR) {
+		    (void) printf("%cREUSEADDR", sep);
+		    opt &= ~PS_SO_REUSEADDR;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_REUSEADDR) */
+
+# if	defined(PS_SO_REUSEPORT)
+		if (opt & PS_SO_REUSEPORT) {
+		    (void) printf("%cREUSEPORT", sep);
+		    opt &= ~PS_SO_REUSEPORT;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_REUSEPORT) */
+
+# if	defined(PS_SO_USELOOPBACK)
+		if (opt & PS_SO_USELOOPBACK) {
+		    (void) printf("%cUSELOOPBACK", sep);
+		    opt &= ~PS_SO_USELOOPBACK;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SO_USELOOPBACK) */
+
+		if (opt)
+		    (void) printf("%cUNKNOWN=%#x", sep, opt);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	if (Ftcptpi & TCPTPI_FLAGS) {
+	    int ss;
+
+	    if ((ss = Lf->lts.ss)) {
+		char sep = ' ';
+
+		if (Ffield)
+		    sep = LSOF_FID_TCPTPI;
+		else if (!ps)
+		    sep = '(';
+		(void) printf("%cSS", sep);
+		ps++;
+		sep = '=';
+
+# if	defined(PS_SS_ASYNC)
+		if (ss & PS_SS_ASYNC) {
+		    (void) printf("%cASYNC", sep);
+		    ss &= ~PS_SS_ASYNC;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ASYNC) */
+
+# if	defined(PS_SS_BOUND)
+		if (ss & PS_SS_BOUND) {
+		    (void) printf("%cBOUND", sep);
+		    ss &= ~PS_SS_BOUND;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_BOUND) */
+
+# if	defined(PS_SS_CANTRCVMORE)
+		if (ss & PS_SS_CANTRCVMORE) {
+		    (void) printf("%cCANTRCVMORE", sep);
+		    ss &= ~PS_SS_CANTRCVMORE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_CANTRCVMORE) */
+
+# if	defined(PS_SS_CANTSENDMORE)
+		if (ss & PS_SS_CANTSENDMORE) {
+		    (void) printf("%cCANTSENDMORE", sep);
+		    ss &= ~PS_SS_CANTSENDMORE;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_CANTSENDMORE) */
+
+# if	defined(PS_SS_ISCONNECTED)
+		if (ss & PS_SS_ISCONNECTED) {
+		    (void) printf("%cISCONNECTED", sep);
+		    ss &= ~PS_SS_ISCONNECTED;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ISCONNECTED) */
+
+# if	defined(PS_SS_ISCONNECTING)
+		if (ss & PS_SS_ISCONNECTING) {
+		    (void) printf("%cISCONNECTING", sep);
+		    ss &= ~PS_SS_ISCONNECTING;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ISCONNECTING) */
+
+# if	defined(PS_SS_ISDISCONNECTI)
+		if (ss & PS_SS_ISDISCONNECTI) {
+		    (void) printf("%cISDISCONNECTI", sep);
+		    ss &= ~PS_SS_ISDISCONNECTI;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_ISDISCONNECTI) */
+
+# if	defined(PS_SS_INTERRUPTED)
+		if (ss & PS_SS_INTERRUPTED) {
+		    (void) printf("%cINTERRUPTED", sep);
+		    ss &= ~PS_SS_INTERRUPTED;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_INTERRUPTED) */
+
+# if	defined(PS_SS_NBIO)
+		if (ss & PS_SS_NBIO) {
+		    (void) printf("%cNBIO", sep);
+		    ss &= ~PS_SS_NBIO;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NBIO) */
+
+# if	defined(PS_SS_NOFDREF)
+		if (ss & PS_SS_NOFDREF) {
+		    (void) printf("%cNOFDREF", sep);
+		    ss &= ~PS_SS_NOFDREF;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NOFDREF) */
+
+# if	defined(PS_SS_NOUSER)
+		if (ss & PS_SS_NOUSER) {
+		    (void) printf("%cNOUSER", sep);
+		    ss &= ~PS_SS_NOUSER;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NOUSER) */
+
+# if	defined(PS_SS_NOWAIT)
+		if (ss & PS_SS_NOWAIT) {
+		    (void) printf("%cNOWAIT", sep);
+		    ss &= ~PS_SS_NOWAIT;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_NOWAIT) */
+
+# if	defined(PS_SS_PRIV)
+		if (ss & PS_SS_PRIV) {
+		    (void) printf("%cPRIV", sep);
+		    ss &= ~PS_SS_PRIV;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_PRIV) */
+
+# if	defined(PS_SS_RCVATMARK)
+		if (ss & PS_SS_RCVATMARK) {
+		    (void) printf("%cRCVATMARK", sep);
+		    ss &= ~PS_SS_RCVATMARK;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_RCVATMARK) */
+
+# if	defined(PS_SS_XOPEN_EXT1)
+		if (ss & PS_SS_XOPEN_EXT1) {
+		    (void) printf("%cXOPEN_EXT1", sep);
+		    ss &= ~PS_SS_XOPEN_EXT1;
+		    sep = ',';
+		}
+# endif	/* defined(PS_SS_XOPEN_EXT1) */
+
+		if (ss)
+		    (void) printf("%cUNKNOWN=%#x", sep, ss);
+		if (Ffield)
+		    putchar(Terminator);
+	    }
+	}
+#endif	/* defined(HASSOSTATE) */
+
+#if	defined(HASTCPTPIW)
 	if (Ftcptpi & TCPTPI_WINDOWS) {
 	    if (Lf->lts.rws) {
 		if (Ffield)
@@ -326,7 +624,7 @@ print_tcptpi(nl)
 		ps++;
 	    }
 	}
-# endif	/* defined(HASTCPTPIW) */
+#endif	/* defined(HASTCPTPIW) */
 
 	if (Ftcptpi && !Ffield && ps)
 	    putchar(')');
@@ -344,8 +642,8 @@ process_socket(f, s)
 	struct pst_fileinfo2 *f;		/* file information */
 	struct pst_socket *s;			/* socket information */
 {
-	int af, err, fp, lp;
 	char buf[1024], tbuf[32];
+	int err, fp, lp;
 	unsigned char *fa = (unsigned char *)NULL;
 	unsigned char *la = (unsigned char *)NULL;
 	size_t len;
@@ -355,6 +653,7 @@ process_socket(f, s)
 	struct sockaddr_in *sa;
 
 #if	defined(HASIPv6)
+	int af;
 	struct sockaddr_in6 *sa6;
 #endif	/* defined(HASIPv6) */
 
@@ -416,6 +715,24 @@ process_socket(f, s)
 	}
 #endif  /* defined(HASTCPTPIQ) */
 
+#if	defined(HASSOOPT)
+/*
+ * Enter socket options.
+ */
+	Lf->lts.opt = (unsigned int)s->pst_options;
+	Lf->lts.ltm = (unsigned int)s->pst_linger;
+	Lf->lts.qlen = (unsigned int)s->pst_qlen;
+	Lf->lts.qlim = (unsigned int)s->pst_qlimit;
+	Lf->lts.qlens = Lf->lts.qlims = (unsigned char)1;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+/*
+ * Enter socket state flags.
+ */
+	Lf->lts.ss = (unsigned int)s->pst_state;
+#endif	/* defined(HASSOSTATE) */
+
 #if	defined(HASTCPTPIW)
 /*
  * Enter window sizes.
@@ -506,12 +823,12 @@ process_socket(f, s)
 	    if ((size_t)s->pst_remaddr_len == sizeof(struct sockaddr_in6)) {
 		sa6 = (struct sockaddr_in6 *)s->pst_remaddr;
 		if ((fp = (int)htons(sa6->sin6_port))
-		||  !IN6_IS_ADDR_UNSPECIFIED(sa6->sin6_addr))
+		||  !IN6_IS_ADDR_UNSPECIFIED(&sa6->sin6_addr))
 		    fa = (unsigned char *)&sa6->sin6_addr;
 	    }
 	    if (la || fa) {
-		if ((la && IN6_IS_ADDR_V4MAPPED(*((struct in6_addr *)la)))
-		||  (fa && IN6_IS_ADDR_V4MAPPED(*((struct in6_addr *)fa))))
+		if ((la && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)la))
+		||  (fa && IN6_IS_ADDR_V4MAPPED((struct in6_addr *)fa)))
 		{
 		    if (la)
 			la = (unsigned char *)IPv6_2_IPv4(la);
@@ -658,11 +975,9 @@ process_stream(f)
 	char *cp;
 	struct l_dev *dp = (struct l_dev *)NULL;
 	int i, ncx, nsn, nsr;
-	mode_t m;
 	size_t nb, nl;
 	KA_T na;
 	static int nsa = 0;
-	SZOFFTYPE rb, wb;
 	dev_t rdev;
 	static struct pst_stream *s = (struct pst_stream *)NULL;
 	static size_t sz = sizeof(struct pst_stream);
