@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.27 2004/12/30 18:41:59 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.28 2005/05/11 12:53:27 abe Exp $";
 #endif
 
 
@@ -122,6 +122,7 @@ struct uxsin {				/* UNIX socket information */
  * Local static values
  */
 
+static char *AX25path = (char *)NULL;	/* path to AX25 /proc information */
 static struct ax25sin **AX25sin = (struct ax25sin **)NULL;
 					/* AX25 socket info, hashed by inode */
 static char *ax25st[] = {
@@ -132,23 +133,32 @@ static char *ax25st[] = {
 	"RECOVERY"			/* 4 */
 };
 #define NAX25ST	(sizeof(ax25st) / sizeof(char *))
+static char *Ipxpath = (char *)NULL;	/* path to IPX /proc information */
 static struct ipxsin **Ipxsin = (struct ipxsin **)NULL;
 					/* IPX socket info, hashed by inode */
+static char *Rawpath = (char *)NULL;	/* path to raw socket /proc
+					 * information */
 static struct rawsin **Rawsin = (struct rawsin **)NULL;
 					/* raw socket info, hashed by inode */
+static char *TCPpath = (char *)NULL;	/* path to TCP /proc information */
 static struct tcp_udp **TcpUdp = (struct tcp_udp **)NULL;
 					/* IPv4 TCP & UDP info, hashed by
 					 * inode */
 
 #if	defined(HASIPv6)
+static char *Raw6path = (char *)NULL;	/* path to raw IPv6 /proc information */
 static struct rawsin **Rawsin6 = (struct rawsin **)NULL;
 					/* IPv6 raw socket info, hashed by
 					 * inode */
+static char *TCP6path = (char *)NULL;	/* path to IPv6 TCP /proc information */
 static struct tcp_udp6 **TcpUdp6 = (struct tcp_udp6 **)NULL;
 					/* IPv6 TCP & UDP info, hashed by
 					 * inode */
+static char *UDP6path = (char *)NULL;	/* path to IPv6 UDP /proc information */
 #endif	/* defined(HASIPv6) */
 
+static char *UDPpath = (char *)NULL;	/* path to UDP /proc information */
+static char *UNIXpath = (char *)NULL;	/* path to UNIX /proc information */
 static struct uxsin **Uxsin = (struct uxsin **)NULL;
 					/* UNIX socket info, hashed by inode */
 
@@ -172,10 +182,11 @@ _PROTOTYPE(static void print_ipxinfo,(struct ipxsin *ip));
 _PROTOTYPE(static void print_rawinfo,(struct rawsin *ip));
 
 #if	defined(HASIPv6)
+_PROTOTYPE(static struct rawsin *check_raw6,(unsigned long i));
+_PROTOTYPE(static struct tcp_udp6 *check_tcpudp6,(unsigned long i, char **p));
 _PROTOTYPE(static void get_raw6,(char *p));
 _PROTOTYPE(static void get_tcpudp6,(char *p, int pr, int clr));
 _PROTOTYPE(static int net6a2in6,(char *as, struct in6_addr *ad));
-_PROTOTYPE(static struct tcp_udp6 *check_tcpudp6,(unsigned long i, char **p));
 #endif	/* defined(HASIPv6) */
 
 
@@ -264,6 +275,26 @@ check_tcpudp(i, p)
 
 
 #if	defined(HASIPv6)
+/*
+ * check_raw6() - check for raw IPv6 socket file
+ */
+
+static struct rawsin *
+check_raw6(i)
+	unsigned long i;		/* socket file's inode number */
+{
+	int h;
+	struct rawsin *rp;
+
+	h = INOHASH(i);
+	for (rp = Rawsin6[h]; rp; rp = rp->next) {
+	    if (i == rp->inode)
+		return(rp);
+	}
+	return((struct rawsin *)NULL);
+}
+
+
 /*
  * check_tcpudp6() - check for IPv6 TCP or UDP socket file
  */
@@ -636,45 +667,6 @@ get_ipx(p)
 }
 
 
-/*
- * get_net() - get /proc/net info
- */
-
-void
-get_net(p, pl)
-	char *p;			/* path to /proc/net/ */
-	int pl;				/* strlen(p) */
-{
-	char buf[MAXPATHLEN], *cp, **fp;
-	FILE *fs;
-	static char *path = (char *)NULL;
-	static int pathl = 0;
-
-	(void) make_proc_path(p, pl, &path, &pathl, "ax25");
-	(void) get_ax25(path);
-	(void) make_proc_path(p, pl, &path, &pathl, "ipx");
-	(void) get_ipx(path);
-	(void) make_proc_path(p, pl, &path, &pathl, "raw");
-	(void) get_raw(path);
-	(void) make_proc_path(p, pl, &path, &pathl, "tcp");
-	(void) get_tcpudp(path, 0, 1);
-	(void) make_proc_path(p, pl, &path, &pathl, "udp");
-	(void) get_tcpudp(path, 1, 0);
-
-#if	defined(HASIPv6)
-	(void) make_proc_path(p, pl, &path, &pathl, "raw6");
-	(void) get_raw6(path);
-	(void) make_proc_path(p, pl, &path, &pathl, "tcp6");
-	(void) get_tcpudp6(path, 0, 1);
-	(void) make_proc_path(p, pl, &path, &pathl, "udp6");
-	(void) get_tcpudp6(path, 1, 0);
-#endif	/* defined(HASIPv6) */
-
-	(void) make_proc_path(p, pl, &path, &pathl, "unix");
-	(void) get_unix(path);
-}
-
-
 /*
  * get_raw() - get /proc/net/raw info
  */
@@ -1376,11 +1368,13 @@ get_unix(p)
 	     * node numbers reported in the stat buffer.
 	     */
 		struct stat sb;
+		int sr;
 
-		if (((HasNFS && !statsafely(path, &sb))
-		||   (!HasNFS && !stat(path, &sb)))
-		&&  ((sb.st_mode & S_IFMT) == S_IFSOCK)
-		) {
+		if (HasNFS)
+		    sr = statsafely(path, &sb);
+		else
+		    sr = stat(path, &sb);
+		if (sr && ((sb.st_mode & S_IFMT) == S_IFSOCK)) {
 		    up->sb_def = 1;
 		    up->sb_dev = sb.st_dev;
 		    up->sb_ino = (unsigned long)sb.st_ino;
@@ -1687,111 +1681,159 @@ process_proc_sock(p, s, ss, l, lss)
 /*
  * Check for socket's inode presence in the protocol info caches.
  */
+	if (AX25path) {
+	    (void) get_ax25(AX25path);
+	    (void) free((FREE_P *)AX25path);
+	    AX25path = (char *)NULL;
+	}
 	if ((ss & SB_INO)
-	&&  (tp = check_tcpudp((unsigned long)s->st_ino, &pr))
+	&&  (ap = check_ax25((unsigned long)s->st_ino))
+	) {
+	
+	/*
+	 * The inode is connected to an AX25 /proc record.
+	 *
+	 * Set the type to "ax25"; save the device name; save the inode number;
+	 * save the destination and source addresses; save the send and receive
+	 * queue sizes; and save the connection state.
+	 */
+	    (void) snpf(Lf->type, sizeof(Lf->type), "ax25");
+	    if (ap->dev_ch)
+		(void) enter_dev_ch(ap->dev_ch);
+	    Lf->inode = ap->inode;
+	    Lf->inp_ty = 1;
+	    print_ax25info(ap);
+	    return;
+	}
+	if (Ipxpath) {
+	    (void) get_ipx(Ipxpath);
+	    (void) free((FREE_P *)Ipxpath);
+	    Ipxpath = (char *)NULL;
+	}
+	if ((ss & SB_INO)
+	&&  (ip = check_ipx((unsigned long)s->st_ino))
 	) {
 
 	/*
-	 * The inode is connected to an IPv4 TCP or UDP /proc record.
+	 * The inode is connected to an IPX /proc record.
 	 *
-	 * Set the type to "inet" or "IPv4"; enter the protocol; put the
-	 * inode number in the DEVICE column in lieu of the PCB address;
-	 * save the local and foreign IPv4 addresses; save the type and
-	 * protocol; and (optionally) save the queue sizes.
+	 * Set the type to "ipx"; enter the inode and device numbers; store
+	 * the addresses, queue sizes, and state in the NAME column.
 	 */
-	    if (Fnet && (FnetTy != 6))
-		Lf->sf |= SELNET;
-
-#if	defined(HASIPv6)
-	    (void) snpf(Lf->type, sizeof(Lf->type), "IPv4");
-#else	/* !defined(HASIPv6) */
-	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
-#endif	/* defined(HASIPv6) */
-
-	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
-	    Lf->inp_ty = 2;
+	    (void) snpf(Lf->type, sizeof(Lf->type), "ipx");
 	    if (ss & SB_INO) {
-		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
-		    (unsigned long)s->st_ino);
-		enter_dev_ch(dev_ch);
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
 	    }
-	    if (tp->faddr || tp->fport) {
-		fs.s_addr = tp->faddr;
-		fa = (unsigned char *)&fs;
-	    } else
-		fa = (unsigned char *)NULL;
-	    if (tp->laddr || tp->lport) {
-		ls.s_addr = tp->laddr;
-		la = (unsigned char *)&ls;
-	    } else
-		la = (unsigned char *)NULL;
-	    ent_inaddr(la, tp->lport, fa, tp->fport, AF_INET);
-	    Lf->lts.type = tp->proto;
-	    Lf->lts.state.i = tp->state;
+	    if (ss & SB_DEV) {
+		Lf->dev = s->st_dev;
+		Lf->dev_def = 1;
+	    }
+	    cp = Namech;
+	    nl = MAXPATHLEN - 2;
+	    if (ip->la && nl) {
 
-#if     defined(HASTCPTPIQ)
-	    Lf->lts.rq = tp->rxq;
-	    Lf->lts.sq = tp->txq;
-	    Lf->lts.rqs = Lf->lts.sqs = 1;
-#endif  /* defined(HASTCPTPIQ) */
+	    /*
+	     * Store the local IPX address.
+	     */
+		len = strlen(ip->la);
+		if (len > nl)
+		    len = nl;
+		(void) strncpy(cp, ip->la, len);
+		cp += len;
+		*cp = '\0';
+		nl -= len;
+	    }
+	    if (ip->ra && nl) {
 
+	    /*
+	     * Store the remote IPX address, prefixed with "->".
+	     */
+		if (nl > 2) {
+		    (void) snpf(cp, nl, "->");
+		    cp += 2;
+		    nl -= 2;
+		}
+		if (nl) {
+		    (void) snpf(cp, nl, "%s", ip->ra);
+		    cp += len;
+		    nl -= len;
+		}
+	    }
+	    (void) print_ipxinfo(ip);
+	    if (Namech[0])
+		enter_nm(Namech);
 	    return;
 	}
-
-#if	defined(HASIPv6)
+	if (Rawpath) {
+	    (void) get_raw(Rawpath);
+	    (void) free((FREE_P *)Rawpath);
+	    Rawpath = (char *)NULL;
+	}
 	if ((ss & SB_INO)
-	&&  (tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))
+	&&  (rp = check_raw((unsigned long)s->st_ino))
 	) {
 
 	/*
-	 * The inode is connected to an IPv6 TCP or UDP /proc record.
+	 * The inode is connected to a raw /proc record.
 	 *
-	 * Set the type to "IPv6"; enter the protocol; put the inode number
-	 * in the DEVICE column in lieu of the PCB address; save the local
-	 * and foreign IPv6 addresses; save the type and protocol; and
-	 * (optionally) save the queue sizes.
+	 * Set the type to "raw"; enter the inode number; store the local
+	 * address, remote address, and state in the NAME column.
 	 */
-	    if (Fnet && (FnetTy != 4))
-		Lf->sf |= SELNET;
-	    (void) snpf(Lf->type, sizeof(Lf->type), "IPv6");
-	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
-	    Lf->inp_ty = 2;
-	    if (ss && SB_INO) {
-		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
-		    (unsigned long)s->st_ino);
-		enter_dev_ch(dev_ch);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "raw");
+	    if (ss & SB_INO) {
+		Lf->inode = (unsigned long)s->st_ino;
+		Lf->inp_ty = 1;
 	    }
-	    af = AF_INET6;
-	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->faddr) || tp6->fport)
-		fa = (unsigned char *)&tp6->faddr;
-	    else
-		fa = (unsigned char *)NULL;
-	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->laddr) || tp6->lport)
-		la = (unsigned char *)&tp6->laddr;
-	    else
-		la = (unsigned char *)NULL;
-	    if ((fa && IN6_IS_ADDR_V4MAPPED(&tp6->faddr))
-	    ||  (la && IN6_IS_ADDR_V4MAPPED(&tp6->laddr))) {
-		af = AF_INET;
-		if (fa)
-		    fa += 12;
-		if (la)
-		    la += 12;
+	    cp = Namech;
+	    nl = MAXPATHLEN - 2;
+	    if (rp->la && rp->lal) {
+
+	    /*
+	     * Store the local raw address.
+	     */
+		if (nl > rp->lal) {
+		    (void) snpf(cp, nl, "%s", rp->la);
+		    cp += rp->lal;
+		    *cp = '\0';
+		    nl -= rp->lal;
+		}
 	    }
-	    ent_inaddr(la, tp6->lport, fa, tp6->fport, af);
-	    Lf->lts.type = tp6->proto;
-	    Lf->lts.state.i = tp6->state;
+	    if (rp->ra && rp->ral) {
 
-#if     defined(HASTCPTPIQ)
-	    Lf->lts.rq = tp6->rxq;
-	    Lf->lts.sq = tp6->txq;
-	    Lf->lts.rqs = Lf->lts.sqs = 1;
-#endif  /* defined(HASTCPTPIQ) */
+	    /*
+	     * Store the remote raw address, prefixed with "->".
+	     */
+		if (nl > (rp->ral + 2)) {
+		    (void) snpf(cp, nl, "->%s", rp->ra);
+		    cp += (rp->ral + 2);
+		    nl -= (rp->ral + 2);
+		}
+	    }
+	    if (rp->sp && rp->spl) {
 
+	    /*
+	     * Store the state, optionally prefixed by a space, in the
+	     * form "st=x...x".
+	     */
+	    
+		if (nl > (len = ((cp == Namech) ? 0 : 1) + 3 + rp->spl)) {
+		    (void) snpf(cp, nl, "%sst=%s",
+			(cp == Namech) ? "" : " ", rp->sp);
+		    cp += len;
+		    *cp = '\0';
+		    nl -= len;
+		}
+	    }
+	    if (Namech[0])
+		enter_nm(Namech);
 	    return;
 	}
-#endif	/* defined(HASIPv6) */
-
+	if (UNIXpath) {
+	    (void) get_unix(UNIXpath);
+	    (void) free((FREE_P *)UNIXpath);
+	    UNIXpath = (char *)NULL;
+	}
 	if ((ss & SB_INO)
 	&&  (up = check_unix((unsigned long)s->st_ino))
 	) {
@@ -1876,91 +1918,25 @@ process_proc_sock(p, s, ss, l, lss)
 	    }
 	    return;
 	}
-	if ((ss & SB_INO)
-	&&  (ap = check_ax25((unsigned long)s->st_ino))
-	) {
-	
-	/*
-	 * The inode is connected to an AX25 /proc record.
-	 *
-	 * Set the type to "ax25"; save the device name; save the inode number;
-	 * save the destination and source addresses; save the send and receive
-	 * queue sizes; and save the connection state.
-	 */
-	    (void) snpf(Lf->type, sizeof(Lf->type), "ax25");
-	    if (ap->dev_ch)
-		(void) enter_dev_ch(ap->dev_ch);
-	    Lf->inode = ap->inode;
-	    Lf->inp_ty = 1;
-	    print_ax25info(ap);
-	    return;
+
+#if	defined(HASIPv6)
+	if (Raw6path) {
+	    if (!Fxopt)
+		(void) get_raw6(Raw6path);
+	    (void) free((FREE_P *)Raw6path);
+	    Raw6path = (char *)NULL;
 	}
-	if ((ss & SB_INO)
-	&&  (ip = check_ipx((unsigned long)s->st_ino))
+	if (!Fxopt && (ss & SB_INO)
+	&&  (rp = check_raw6((unsigned long)s->st_ino))
 	) {
 
 	/*
-	 * The inode is connected to an IPX /proc record.
+	 * The inode is connected to a raw IPv6 /proc record.
 	 *
-	 * Set the type to "ipx"; enter the inode and device numbers; store
-	 * the addresses, queue sizes, and state in the NAME column.
-	 */
-	    (void) snpf(Lf->type, sizeof(Lf->type), "ipx");
-	    if (ss & SB_INO) {
-		Lf->inode = (unsigned long)s->st_ino;
-		Lf->inp_ty = 1;
-	    }
-	    if (ss & SB_DEV) {
-		Lf->dev = s->st_dev;
-		Lf->dev_def = 1;
-	    }
-	    cp = Namech;
-	    nl = MAXPATHLEN - 2;
-	    if (ip->la && nl) {
-
-	    /*
-	     * Store the local IPX address.
-	     */
-		len = strlen(ip->la);
-		if (len > nl)
-		    len = nl;
-		(void) strncpy(cp, ip->la, len);
-		cp += len;
-		*cp = '\0';
-		nl -= len;
-	    }
-	    if (ip->ra && nl) {
-
-	    /*
-	     * Store the remote IPX address, prefixed with "->".
-	     */
-		if (nl > 2) {
-		    (void) snpf(cp, nl, "->");
-		    cp += 2;
-		    nl -= 2;
-		}
-		if (nl) {
-		    (void) snpf(cp, nl, "%s", ip->ra);
-		    cp += len;
-		    nl -= len;
-		}
-	    }
-	    (void) print_ipxinfo(ip);
-	    if (Namech[0])
-		enter_nm(Namech);
-	    return;
-	}
-	if ((ss & SB_INO)
-	&&  (rp = check_raw((unsigned long)s->st_ino))
-	) {
-
-	/*
-	 * The inode is connected to a raw /proc record.
-	 *
-	 * Set the type to "raw"; enter the inode number; store the local
+	 * Set the type to "raw6"; enter the inode number; store the local
 	 * address, remote address, and state in the NAME column.
 	 */
-	    (void) snpf(Lf->type, sizeof(Lf->type), "raw");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "raw6");
 	    if (ss & SB_INO) {
 		Lf->inode = (unsigned long)s->st_ino;
 		Lf->inp_ty = 1;
@@ -1970,7 +1946,7 @@ process_proc_sock(p, s, ss, l, lss)
 	    if (rp->la && rp->lal) {
 
 	    /*
-	     * Store the local raw address.
+	     * Store the local raw IPv6 address.
 	     */
 		if (nl > rp->lal) {
 		    (void) snpf(cp, nl, "%s", rp->la);
@@ -2009,6 +1985,133 @@ process_proc_sock(p, s, ss, l, lss)
 		enter_nm(Namech);
 	    return;
 	}
+	if (TCP6path) {
+	    if (!Fxopt)
+		(void) get_tcpudp6(TCP6path, 0, 1);
+	    (void) free((FREE_P *)TCP6path);
+	    TCP6path = (char *)NULL;
+	}
+	if (UDP6path) {
+	    if (!Fxopt)
+		(void) get_tcpudp6(UDP6path, 1, 0);
+	    (void) free((FREE_P *)UDP6path);
+	    UDP6path = (char *)NULL;
+	}
+	if (!Fxopt && (ss & SB_INO)
+	&&  (tp6 = check_tcpudp6((unsigned long)s->st_ino, &pr))
+	) {
+
+	/*
+	 * The inode is connected to an IPv6 TCP or UDP /proc record.
+	 *
+	 * Set the type to "IPv6"; enter the protocol; put the inode number
+	 * in the DEVICE column in lieu of the PCB address; save the local
+	 * and foreign IPv6 addresses; save the type and protocol; and
+	 * (optionally) save the queue sizes.
+	 */
+	    if (Fnet && (FnetTy != 4))
+		Lf->sf |= SELNET;
+	    (void) snpf(Lf->type, sizeof(Lf->type), "IPv6");
+	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
+	    Lf->inp_ty = 2;
+	    if (ss && SB_INO) {
+		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
+		    (unsigned long)s->st_ino);
+		enter_dev_ch(dev_ch);
+	    }
+	    af = AF_INET6;
+	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->faddr) || tp6->fport)
+		fa = (unsigned char *)&tp6->faddr;
+	    else
+		fa = (unsigned char *)NULL;
+	    if (!IN6_IS_ADDR_UNSPECIFIED(&tp6->laddr) || tp6->lport)
+		la = (unsigned char *)&tp6->laddr;
+	    else
+		la = (unsigned char *)NULL;
+	    if ((fa && IN6_IS_ADDR_V4MAPPED(&tp6->faddr))
+	    ||  (la && IN6_IS_ADDR_V4MAPPED(&tp6->laddr))) {
+		af = AF_INET;
+		if (fa)
+		    fa += 12;
+		if (la)
+		    la += 12;
+	    }
+	    ent_inaddr(la, tp6->lport, fa, tp6->fport, af);
+	    Lf->lts.type = tp6->proto;
+	    Lf->lts.state.i = tp6->state;
+
+#if     defined(HASTCPTPIQ)
+	    Lf->lts.rq = tp6->rxq;
+	    Lf->lts.sq = tp6->txq;
+	    Lf->lts.rqs = Lf->lts.sqs = 1;
+#endif  /* defined(HASTCPTPIQ) */
+
+	    return;
+	}
+#endif	/* defined(HASIPv6) */
+
+	if (TCPpath) {
+	    if (!Fxopt)
+		(void) get_tcpudp(TCPpath, 0, 1);
+	    (void) free((FREE_P *)TCPpath);
+	    TCPpath = (char *)NULL;
+	}
+	if (UDPpath) {
+	    if (!Fxopt)
+		(void) get_tcpudp(UDPpath, 1, 0);
+	    (void) free((FREE_P *)UDPpath);
+	    UDPpath = (char *)NULL;
+	}
+	if (!Fxopt && (ss & SB_INO)
+	&&  (tp = check_tcpudp((unsigned long)s->st_ino, &pr))
+	) {
+
+	/*
+	 * The inode is connected to an IPv4 TCP or UDP /proc record.
+	 *
+	 * Set the type to "inet" or "IPv4"; enter the protocol; put the
+	 * inode number in the DEVICE column in lieu of the PCB address;
+	 * save the local and foreign IPv4 addresses; save the type and
+	 * protocol; and (optionally) save the queue sizes.
+	 */
+	    if (Fnet && (FnetTy != 6))
+		Lf->sf |= SELNET;
+
+#if	defined(HASIPv6)
+	    (void) snpf(Lf->type, sizeof(Lf->type), "IPv4");
+#else	/* !defined(HASIPv6) */
+	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
+#endif	/* defined(HASIPv6) */
+
+	    (void) snpf(Lf->iproto, sizeof(Lf->iproto), "%.*s", IPROTOL-1, pr);
+	    Lf->inp_ty = 2;
+	    if (ss & SB_INO) {
+		(void) snpf(dev_ch, sizeof(dev_ch), "%lu",
+		    (unsigned long)s->st_ino);
+		enter_dev_ch(dev_ch);
+	    }
+	    if (tp->faddr || tp->fport) {
+		fs.s_addr = tp->faddr;
+		fa = (unsigned char *)&fs;
+	    } else
+		fa = (unsigned char *)NULL;
+	    if (tp->laddr || tp->lport) {
+		ls.s_addr = tp->laddr;
+		la = (unsigned char *)&ls;
+	    } else
+		la = (unsigned char *)NULL;
+	    ent_inaddr(la, tp->lport, fa, tp->fport, AF_INET);
+	    Lf->lts.type = tp->proto;
+	    Lf->lts.state.i = tp->state;
+
+#if     defined(HASTCPTPIQ)
+	    Lf->lts.rq = tp->rxq;
+	    Lf->lts.sq = tp->txq;
+	    Lf->lts.rqs = Lf->lts.sqs = 1;
+#endif  /* defined(HASTCPTPIQ) */
+
+	    return;
+	}
 /*
  * The socket's protocol can't be identified.
  */
@@ -2021,5 +2124,42 @@ process_proc_sock(p, s, ss, l, lss)
 	    Lf->dev = s->st_dev;
 	    Lf->dev_def = 1;
 	}
-	enter_nm("can't identify protocol");
+	enter_nm(Fxopt ? "can't identify protocol (-X specified)"
+		       : "can't identify protocol");
+}
+
+
+/*
+ * set_net_paths() - set /proc/net paths
+ */
+
+void
+set_net_paths(p, pl)
+	char *p;			/* path to /proc/net/ */
+	int pl;				/* strlen(p) */
+{
+	int pathl;
+
+	pathl = 0;
+	(void) make_proc_path(p, pl, &AX25path, &pathl, "ax25");
+	pathl = 0;
+	(void) make_proc_path(p, pl, &Ipxpath, &pathl, "ipx");
+	pathl = 0;
+	(void) make_proc_path(p, pl, &Rawpath, &pathl, "raw");
+	pathl = 0;
+	(void) make_proc_path(p, pl, &TCPpath, &pathl, "tcp");
+	pathl = 0;
+	(void) make_proc_path(p, pl, &UDPpath, &pathl, "udp");
+
+#if	defined(HASIPv6)
+	pathl = 0;
+	(void) make_proc_path(p, pl, &Raw6path, &pathl, "raw6");
+	pathl = 0;
+	(void) make_proc_path(p, pl, &TCP6path, &pathl, "tcp6");
+	pathl = 0;
+	(void) make_proc_path(p, pl, &UDP6path, &pathl, "udp6");
+#endif	/* defined(HASIPv6) */
+
+	pathl = 0;
+	(void) make_proc_path(p, pl, &UNIXpath, &pathl, "unix");
 }
