@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.23 2004/03/10 23:51:11 abe Exp $
+ * $Id: machine.h,v 1.26 2005/05/11 12:53:27 abe Exp $
  */
 
 
@@ -219,6 +219,14 @@
 /* #define SETLFILEADD Lf->... */
 
 
+/*
+ * HASMNTSUP is defined for those dialects that support the mount supplement
+ * option.
+ */
+
+#define	HASMNTSUP	1
+
+
 /*
  * HASMOPT is defined for those dialects that support the reading of
  * kernel memory from an alternate file.
@@ -391,9 +399,16 @@
 /*
  * HASSETLOCALE is defined for those dialects that have <locale.h> and
  * setlocale().
+ *
+ * If the dialect also has wide character support for language locales,
+ * HASWIDECHAR activates lsof's wide character support and WIDECHARINCL
+ * defines the header file (if any) that must be #include'd to use the
+ * mblen() and mbtowc() functions.
  */
 
 #define	HASSETLOCALE	1
+#define	HASWIDECHAR	1
+/* #define	WIDECHARINCL	<wchar.h>	*/
 
 
 /*
@@ -478,8 +493,8 @@
  * option's default binary value -- 0 or 1.
  */
 
-/* #define	HASXOPT		"help text for X option" */
-/* #define	HASXOPT_VALUE	1 */
+#define	HASXOPT		"skip TCP&UDP files"
+#define	HASXOPT_VALUE	0
 
 
 /*
