@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.22 2002/04/19 12:11:10 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.24 2004/12/30 18:42:24 abe Exp $";
 #endif
 
 
@@ -79,9 +79,9 @@ process_socket(sa)
 #define	UNPADDR_IN_MBUF
 
 #if	defined(NETBSDV)
-# if	NETBSDV>=1040
+# if	NETBSDV>=1004000
 #undef	UNPADDR_IN_MBUF
-# endif	/* NETBSDV>=1040 */
+# endif	/* NETBSDV>=1004000 */
 #endif	/* defined(NETBSDV) */
 
 #if	defined(UNPADDR_IN_MBUF)
@@ -141,6 +141,21 @@ process_socket(sa)
 	Lf->lts.rqs = Lf->lts.sqs = 1;
 #endif	/* defined(HASTCPTPIQ) */
 
+#if	defined(HASSOOPT)
+	Lf->lts.ltm = (unsigned int)s.so_linger;
+	Lf->lts.opt = (unsigned int)s.so_options;
+	Lf->lts.qlen = (unsigned int)s.so_qlen;
+	Lf->lts.qlim = (unsigned int)s.so_qlimit;
+	Lf->lts.rbsz = (unsigned long)s.so_rcv.sb_mbmax;
+	Lf->lts.sbsz = (unsigned long)s.so_snd.sb_mbmax;
+	Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
+		      = Lf->lts.sbszs = (unsigned char)1;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	Lf->lts.ss = (unsigned int)s.so_state;
+#endif	/* defined(HASSOSTATE) */
+
 /*
  * Process socket by the associated domain family.
  */
@@ -286,6 +301,18 @@ process_socket(sa)
 	    if (ta && !kread(ta, (char *)&t, sizeof(t))) {
 		Lf->lts.type = 0;
 		Lf->lts.state.i = (int)t.t_state;
+
+#if	defined(HASTCPOPT)
+# if	defined(OPENBSDV)
+		Lf->lts.mss = (unsigned long)t.t_maxseg;
+# else	/* !defined(OPENSDV) */
+		Lf->lts.mss = (unsigned long)t.t_ourmss;
+# endif	/* defined(OPENSDV) */
+
+		Lf->lts.msss = (unsigned char)1;
+		Lf->lts.topt = (unsigned int)t.t_flags;
+#endif	/* defined(HASTCPOPT) */
+
 	    }
 	    break;
 /*
