@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.35 2003/10/13 13:30:04 abe Exp $
+ * $Id: dlsof.h,v 1.40 2004/12/30 18:42:59 abe Exp $
  */
 
 
@@ -50,6 +50,13 @@
 
 #include <stdlib.h>
 #include <dirent.h>
+
+# if	defined(HASZONES)
+#define	_KERNEL
+#include<sys/zone.h>
+#undef	_KERNEL
+# endif	/* defined(HASZONES) */
+
 #include <kvm.h>
 #include <nlist.h>
 #include <signal.h>
@@ -92,11 +99,17 @@
 # endif	/* solaris>=80000 */
 
 # if	defined(HAS_IPCLASSIFIER_H)
+#define	ffs		__kernel_ffs
 #define	inet_ntop	__inet_ntop
 #define	inet_pton	__inet_pton
+#define	longjmp		__kernel_longjmp
+#define	setjmp		__kernel_setjmp
 #include <inet/ipclassifier.h>
+#undef	ffs
 #undef	inet_ntop
-#undef	inet_pto
+#undef	inet_pton
+#undef	longjmp
+#undef	setjmp
 # endif	/* defined(HAS_IPCLASSIFIER_H) */
 
 #include <inet/ip.h>
@@ -159,13 +172,21 @@
 #include <sys/fs/autofs.h>
 
 #  if	solaris>=100000
+#define	printf		lsof_printf
+#define	snprintf	lsof_snprintf
+#define	sprintf		lsof_sprintf
 #define	swab		lsof_swab
 #define	vprintf		lsof_vprintf
+#define	vsnprintf	lsof_vsnprintf
 #define	vsprintf	lsof_vsprintf
 #include <sys/fs/dv_node.h>
-#undef	lsof_swab
-#undef	lsof_vprintf
-#undef	lsof_vsprintf
+#undef	printf
+#undef	snprintf
+#undef	sprintf
+#undef	swab
+#undef	vprintf
+#undef	vsnprintf
+#undef	vsprintf
 #  endif	/* solaris>=100000 */
 
 #include <sys/door.h>
@@ -232,7 +253,23 @@ struct lock_descriptor {
 #include <sys/fs/snode.h>
 #include <sys/fs/tmpnode.h>
 #include <nfs/nfs.h>
+
+# if	solaris>=100000
+#define	_KERNEL
+# endif	/* solaris >= 100000 */
+
 #include <nfs/rnode.h>
+
+#  if	solaris>=100000
+#include <nfs/mount.h>
+#include <nfs/nfs4.h>
+#include <nfs/rnode4.h>
+#  endif	/* solaris>=100000 */
+
+# if	solaris>=100000
+#undef	_KERNEL
+# endif	/* solaris >= 100000 */
+
 #include <sys/proc.h>
 #include <sys/user.h>
 
@@ -269,6 +306,7 @@ extern int nlist();
 #define	AFSAPATHDEF	"/usr/vice/etc/modload/libafs"
 # endif	/* defined(HAS_AFS) && !defined(AFSAPATHDEF) */
 
+#define	ALLKMEM		"/dev/allkmem"
 #define	COMP_P		const void
 #define	CWDLEN		(MAXPATHLEN+1)
 #define DEVINCR		1024		/* device table malloc() increment */
@@ -309,7 +347,8 @@ typedef	void *		KA_T;
 #define	PR_ROOTINO	2		/* root inode for proc file system */
 #  endif	/* defined(HASPROCFS) */
 
-#define	PROCDFLT	256		/* default size for local proc table */
+#define	PROCDFLT	256		/* default size for local proc table --
+					 * MUST BE > 4!!!  */
 #define PROCSIZE	sizeof(struct proc)
 #define	PROCTRYLM	5		/* times to try to read proc table */
 #define QSORT_P		char
@@ -366,6 +405,7 @@ extern struct clone *Clone;
 extern	major_t	CloneMaj;
 extern char **Fsinfo;
 extern int Fsinfomax;
+extern int HasALLKMEM;
 extern int HaveCloneMaj;
 extern kvm_t *Kd;
 
