@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.46 2003/10/13 13:30:04 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.49 2004/12/30 18:42:59 abe Exp $";
 #endif
 
 
@@ -86,6 +86,11 @@ _PROTOTYPE(static int read_nin,(KA_T na, KA_T ia, struct inode *i));
 _PROTOTYPE(static int read_nmn,(KA_T na, KA_T ia, struct mvfsnode *m));
 _PROTOTYPE(static int read_npn,(KA_T na, KA_T pa, struct pcnode *p));
 _PROTOTYPE(static int read_nrn,(KA_T na, KA_T ra, struct rnode *r));
+
+#if	solaris>=100000
+_PROTOTYPE(static int read_nrn4,(KA_T na, KA_T ra, struct rnode4 *r));
+#endif	/* solaris>=100000 */
+
 _PROTOTYPE(static int read_nsn,(KA_T na, KA_T sa, struct snode *s));
 _PROTOTYPE(static int read_ntn,(KA_T na, KA_T ta, struct tmpnode *t));
 _PROTOTYPE(static int read_nvn,(KA_T na, KA_T va, struct vnode *v));
@@ -136,6 +141,8 @@ static KA_T Prvops;		/* prvnodeops address */
 static KA_T Sam1vops;		/* samfs_vnodeops address */
 static KA_T Sam2vops;		/* samfs_client_vnodeops address */
 static KA_T Sckvops;		/* sock_vnodeops address */
+static KA_T Sncavops;		/* socknca_vnodeops address */
+static KA_T Stpivops;		/* socktpi_vnodeops address */
 static KA_T Spvops;		/* spec_vnodeops address */
 static KA_T Tvops;		/* tmp_vnodeops address */
 static KA_T Uvops;		/* ufs_vnodeops address */
@@ -324,7 +331,7 @@ isvlocked(va)
 	KA_T fp;
 #endif	/* solaris<20500 */
 
-	int l;
+	int i, l;
 
 #if	solaris>=20300
 	struct lock_descriptor ld;
@@ -357,9 +364,11 @@ isvlocked(va)
 
 	{
 	    ff = fp = (KA_T)va->v_filocks;
+	    i = 0;
 	    do {
 		if (kread(fp, (char *)&f, sizeof(f)))
 		    return(' ');
+		i++;
 		if (f.set.l_pid != (pid_t)Lp->pid)
 		    continue;
 		if (f.set.l_whence == 0 && f.set.l_start == 0
@@ -377,15 +386,17 @@ isvlocked(va)
 		default:
 		    return('N');
 		}
-	    } while ((fp = (KA_T)f.next) && fp != ff);
+	    } while ((fp = (KA_T)f.next) && (fp != ff) && (i < 10000));
 	}
 #endif	/* solaris<20500 */
 
 #if	solaris>=20300
 	lf = lp = (KA_T)va->v_filocks;
+	i = 0;
 	do {
 	    if (kread(lp, (char *)&ld, sizeof(ld)))
 		return(' ');
+	    i++;
 	    if (!(LOCK_FLAGS & ACTIVE_LOCK) || LOCK_OWNER != (pid_t)Lp->pid)
 		continue;
 	    if (LOCK_START == 0
@@ -411,7 +422,7 @@ isvlocked(va)
 	    default:
 		return('L');
 	    }
-	} while ((lp = (KA_T)LOCK_NEXT) && lp != lf);
+	} while ((lp = (KA_T)LOCK_NEXT) && (lp != lf) && (i < 10000));
 	return(' ');
 #endif	/* solaris>=20300 */
 
@@ -611,6 +622,7 @@ process_node(va)
 	struct dv_node dv;
 	unsigned char dv_devs = 0;
 	unsigned char dvs = 0;
+	struct rnode4 r4;
 #endif	/* solaris<100000 */
 
 
@@ -659,6 +671,8 @@ process_node(va)
 	    GETVOPS("sam1vops", Drive_Nl, Sam1vops);
 	    GETVOPS("sam2vops", Drive_Nl, Sam2vops);
 	    GETVOPS("sckvops", Drive_Nl, Sckvops);
+	    GETVOPS("sncavops", Drive_Nl, Sncavops);
+	    GETVOPS("stpivops", Drive_Nl, Stpivops);
 	    GETVOPS("spvops", Drive_Nl, Spvops);
 	    GETVOPS("tvops", Drive_Nl, Tvops);
 	    GETVOPS("uvops", Drive_Nl, Uvops);
@@ -716,6 +730,10 @@ process_node(va)
 	Lf->fsv |= FSV_NI;
 #endif	/* defined(HASFSTRUCT) */
 
+#if	defined(HASLFILEADD) && defined(HAS_V_PATH)
+	Lf->V_path = (KA_T)v->v_path;
+#endif	/* defined(HASLFILEADD) && defined(HAS_V_PATH) */
+
 	vs = (KA_T)v->v_stream;
 /*
  * Check for a Solaris socket.
@@ -762,20 +780,12 @@ vfs_read_error:
 		Lf->is_stream = 1;
 	    }
 	    if (Ntype < 0) {
-
-#if	solaris<100000
 		(void) snpf(Namech, Namechl,
 		    "unknown file system type%s%s%s, v_op: %s",
 		    fxs ? " (" : "",
 		    fxs ? Fsinfo[fx] : "",
 		    fxs ? ")" : "",
 		    print_kptr((KA_T)v->v_op, (char *)NULL, 0));
-#else	/* solaris>=100000 */
-		(void) snpf(Namech, Namechl,
-		    "unknown file system type: %s",
-		    fxs ? Fsinfo[fx] : "(unknown)");
-#endif	/* solaris<100000 */
-
 		enter_nm(Namech);
 		return;
 	    }
@@ -948,6 +958,14 @@ vfs_read_error:
 	    if (read_nrn(va, (KA_T)v->v_data, &r))
 		return;
 	    break;
+
+#if	solaris>=100000
+	case N_NFS4:
+	    if (read_nrn4(va, (KA_T)v->v_data, &r4))
+		return;
+	    break;
+#endif	/* solaris>=100000 */
+
 	case N_NM:
 	    if (nns)
 		realvp = (KA_T)nn.nm_filevp;
@@ -1233,6 +1251,14 @@ vfs_read_error:
 		if (read_nrn(va, (KA_T)v->v_data, &r))
 		    return;
 		break;
+
+#if	solaris>=100000
+	    case N_NFS4:
+		if (read_nrn4(va, (KA_T)v->v_data, &r4))
+		    return;
+		break;
+#endif	/* solaris>=100000 */
+
 	    case N_NM:
 		if (read_nnn(va, (KA_T)v->v_data, &nn))
 		    return;
@@ -1525,6 +1551,14 @@ vfs_read_error:
 	    dev = r.r_attr.va_fsid;
 	    devs = 1;
 	    break;
+
+#if	solaris>=100000
+	case N_NFS4:
+	    dev = r4.r_attr.va_fsid;
+	    devs = 1;
+	    break;
+#endif	/* solaris>=100000 */
+
 	case N_NM:
 	    if (nns) {
 		dev = (dev_t)nn.nm_vattr.va_fsid;
@@ -1748,13 +1782,7 @@ vfs_read_error:
 	    Lf->inp_ty = 1;
 	    break;
 	case N_HSFS:
-
-#if	defined(HAS_HS_NODEID)
 	    Lf->inode = (unsigned long)h.hs_nodeid;
-#else	/* defined(HAS_HS_NODEID) */
-	    Lf->inode = (unsigned long)h.hs_dirent.ext_lbn;
-#endif	/* defined(HAS_HS_NODEID) */
-
 	    Lf->inp_ty = 1;
 	    break;
 
@@ -1776,6 +1804,14 @@ vfs_read_error:
 	    Lf->inode = (unsigned long)r.r_attr.va_nodeid;
 	    Lf->inp_ty = 1;
 	    break;
+
+#if	solaris>=100000
+	case N_NFS4:
+	    Lf->inode = (unsigned long)r4.r_attr.va_nodeid;
+	    Lf->inp_ty = 1;
+	    break;
+#endif	/* solaris>=100000 */
+
 	case N_NM:
 	    Lf->inode = (unsigned long)nn.nm_vattr.va_nodeid;
 	    Lf->inp_ty = 1;
@@ -1956,6 +1992,17 @@ vfs_read_error:
 		}
 		break;
 
+#if	solaris>=100000
+	    case N_NFS4:
+		if ((type == VCHR || type == VBLK) && !Fsize)
+		    Lf->off_def = 1;
+		else {
+		    Lf->sz = (SZOFFTYPE)r4.r_size;
+		    Lf->sz_def = 1;
+		}
+		break;
+#endif	/* solaris>=100000 */
+
 	    case N_PCFS:
 		Lf->sz = (SZOFFTYPE)pc.pc_size;
 		Lf->sz_def = 1;
@@ -2076,6 +2123,14 @@ vfs_read_error:
 		Lf->nlink = (long)r.r_attr.va_nlink;
 		Lf->nlink_def = 1;
 		break;
+
+#if	solaris>=100000
+	    case N_NFS4:
+		Lf->nlink = (long)r4.r_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+#endif	/* solaris>=100000 */
+
 	    case N_PCFS:
 		break;
 
@@ -2113,8 +2168,10 @@ vfs_read_error:
 /*
  * Record an NFS selection.
  */
-	if (Ntype == N_NFS && Fnfs)
-	    Lf->sf |= SELNFS;
+	if (Fnfs) {
+	    if ((Ntype == N_NFS) || (Ntype == N_NFS4))
+		Lf->sf |= SELNFS;
+	}
 
 #if	solaris>=20500
 /*
@@ -2158,15 +2215,17 @@ vfs_read_error:
 		    (void) completevfs(nvfs, &dev);
 		}
 
-#if	defined(HASNCACHE)
+# if	defined(HASNCACHE)
 		if (nvfs && nvfs->dir && nvfs->fsname) {
 		    fa = (char *)NULL;
 		    vfs = nvfs;
 		}
-#endif	/* defined(HASNCACHE) */
+# endif	/* defined(HASNCACHE) */
 
 	    }
 	}
+
+# if	defined(HASNCACHE)
 /*
  * If there's a namenode and its device and node number match this one,
  * use the nm_mountpt's address for name cache lookups.
@@ -2174,6 +2233,7 @@ vfs_read_error:
 	if (nns && devs && (dev == nn.nm_vattr.va_fsid) && (Lf->inp_ty == 1)
 	&&  (Lf->inode == (unsigned long)nn.nm_vattr.va_nodeid))
 	    Lf->na = (KA_T)nn.nm_mountpt;
+# endif	/* defined(HASNCACHE) */
 #endif	/* solaris>=20500 */
 
 /*
@@ -2182,11 +2242,22 @@ vfs_read_error:
 	if (vfs) {
 	    Lf->fsdir = vfs->dir;
 	    Lf->fsdev = vfs->fsname;
-	    if (!Lf->fsdir && !Lf->fsdev && kvs && fxs)
+	    if (!Lf->fsdir && !Lf->fsdev && kvs && fxs) {
+
+	    /*
+	     * The file system names are unknown.
+	     *
+	     * Set the file system device to the file system type and clear
+	     * the doubtful device numbers.
+	     */
 		Lf->fsdev = Fsinfo[fx];
+		devs = 0;
+		rdevs = 0;
+	    }
 
 #if	defined(HASFSINO)
-	    Lf->fs_ino = vfs->fs_ino;
+	    else
+		Lf->fs_ino = vfs->fs_ino;
 #endif	/* defined(HASFSINO) */
 
 	}
@@ -2645,7 +2716,7 @@ read_mi(s, rdev, so, so_st, so_ad, sdp)
 	mn[nl] = '\0';
 	qp = (KA_T)sd.sd_wrq;
 	for (i = 0; qp && i < 20; i++, qp = (KA_T)q.q_next) {
-	    if (!qp ||  kread(qp, (char *)&q, sizeof(q)))
+	    if (!qp || kread(qp, (char *)&q, sizeof(q)))
 		break;
 	    if ((ka = (KA_T)q.q_qinfo) == (KA_T)NULL
 	    ||  kread(ka, (char *)&qi, sizeof(qi)))
@@ -3288,6 +3359,34 @@ read_nrn(na, ra, r)
 }
 
 
+#if	solaris>=100000
+/*
+ * read_nrn4() - read node's rnode4
+ */
+
+static int
+read_nrn4(na, ra, r)
+	KA_T na;			/* containing node's address */
+	KA_T ra;			/* rnode address */
+	struct rnode4 *r;		/* rnode receiver */
+{
+	char tbuf[32];
+
+	if (!ra ||
+	    kread((KA_T)ra, (char *)r, sizeof(struct rnode4))
+	) {
+	    (void) snpf(Namech, Namechl, "node at %s: can't read rnode4: %s",
+		print_kptr(na, tbuf, sizeof(tbuf)),
+		print_kptr(ra, (char *)NULL, 0));
+	    enter_nm(Namech);
+	    return(1);
+	}
+	return(0);
+}
+#endif	/* solaris>=100000 */
+
+
+
 #if	solaris>=20600
 /*
  * read_nson() - read node's sonode
@@ -3603,7 +3702,12 @@ vop2ty(vp)
 	else if (N3vops && N3vops == (KA_T)vp->v_op)
 	    return(N_NFS);
 	else if (N4vops && N4vops == (KA_T)vp->v_op)
+
+# if	solaris>=100000
+	    return(N_NFS4);
+# else	/* solaris<100000 */
 	    return(N_NFS);
+# endif	/* solaris>=100000 */
 
 # if	defined(HASVXFS)
 	for (i = 0; i < VXVOP_NUM; i++) {
@@ -3647,8 +3751,12 @@ vop2ty(vp)
 	    return(N_SAMFS);
 	else if (Sam2vops && Sam2vops == (KA_T)vp->v_op)
 	    return(N_SAMFS);
-	else if (Sckvops && Sckvops == (KA_T)vp->v_op)
+	else if ((Sckvops  && Sckvops  == (KA_T)vp->v_op)
+	     ||  (Sncavops && Sncavops == (KA_T)vp->v_op)
+	     ||  (Stpivops && Stpivops == (KA_T)vp->v_op)
+	) {
 	    return(N_SOCK);
+	}
 
 # if	defined(HAS_AFS)
 /*
