@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.30 2004/03/10 23:52:32 abe Exp $
+ * $Id: machine.h,v 1.32 2004/10/17 21:56:37 abe Exp $
  */
 
 
@@ -226,6 +226,14 @@
 /* #define SETLFILEADD Lf->... */
 
 
+/*
+ * HASMNTSUP is defined for those dialects that support the mount supplement
+ * option.
+ */
+
+/* #define	HASMNTSUP	1	*/
+
+
 /*
  * HASMOPT is defined for those dialects that support the reading of
  * kernel memory from an alternate file.
@@ -401,10 +409,21 @@
 /*
  * HASSETLOCALE is defined for those dialects that have <locale.h> and
  * setlocale().
+ *
+ * If the dialect also has wide character support for language locales,
+ * HASWIDECHAR activates lsof's wide character support and WIDECHARINCL
+ * defines the header file (if any) that must be #include'd to use the
+ * mblen() and mbtowc() functions.
  */
 
 #define	HASSETLOCALE	1
 
+# if	UNIXWAREV>=70104
+#define	HASWIDECHAR	1
+# endif	/* UNIXWAREV>=70104 */
+
+/* #define	WIDECHARINCL	<wchar.h>	*/
+
 
 /*
  * HASSNODE is defined for those dialects that have snodes.
