@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.38 2005/01/05 15:38:21 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.39 2005/05/11 13:02:46 abe Exp $";
 #endif
 
 
@@ -140,6 +140,11 @@ alloc_lfile(nm, num)
 		(void) free((FREE_P *)Lf->nm);
 	    if (Lf->nma)
 		(void) free((FREE_P *)Lf->nma);
+
+#if	defined(HASLFILEADD) && defined(CLRLFILEADD)
+	    CLRLFILEADD(Lf)
+#endif	/* defined(HASLFILEADD) && defined(CLRLFILEADD) */
+
 /*
  * Othwerise, allocate a new structure.
  */
@@ -519,6 +524,11 @@ free_lproc(lp)
 		(void) free((FREE_P *)lf->nma);
 		lf->nma = (char *)NULL;
 	    }
+
+#if	defined(HASLFILEADD) && defined(CLRLFILEADD)
+	    CLRLFILEADD(lf)
+#endif	/* defined(HASLFILEADD) && defined(CLRLFILEADD) */
+
 	    nf = lf->next;
 	    (void) free((FREE_P *)lf);
 	}
@@ -639,11 +649,11 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 #endif	/* defined(HASSECURITY) */
 
 /*
- * If the excluding of process listing by UID has been specified,
- * see if the owner of this process is excluded.
+ * If the excluding of process listing by UID has been specified, see if the
+ * owner of this process is excluded.
  */
 	if (Nuidexcl) {
-	    for (i = j = 0; i < Nuid && j < Nuidexcl; i++) {
+	    for (i = j = 0; (i < Nuid) && (j < Nuidexcl); i++) {
 		if (!Suid[i].excl)
 		    continue;
 		if (Suid[i].uid == (uid_t)uid)
@@ -651,6 +661,32 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 		j++;
 	    }
 	}
+/*
+ * If the excluding of process listing by PGID has been specified, see if this
+ * PGID is excluded.
+ */
+	if (Npgidx) {
+	    for (i = j = 0; (i < Npgid) && (j < Npgidx); i++) {
+		if (!Spgid[i].x)
+		    continue;
+		if (Spgid[i].i == pgid)
+		    return(1);
+		j++;
+	    }
+	}
+/*
+ * If the excluding of process listing by PID has been specified, see if this
+ * PID is excluded.
+ */
+	if (Npidx) {
+	    for (i = j = 0; (i < Npid) && (j < Npidx); i++) {
+		if (!Spid[i].x)
+		    continue;
+		if (Spid[i].i == pid)
+		    return(1);
+		j++;
+	    }
+	}
 /*
  * If the listing of all processes is selected, then this one is not excluded.
  *
@@ -671,10 +707,12 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 	}
 /*
  * If the listing of processes has been specified by process group ID, see
- * if this one is specified.
+ * if this one is included or excluded.
  */
-	if (Npgid && (Selflags & SELPGID)) {
-	    for (i = 0; i < Npgid; i++) {
+	if (Npgidi && (Selflags & SELPGID)) {
+	    for (i = j = 0; (i < Npgid) && (j < Npgidi); i++) {
+		if (Spgid[i].x)
+		    continue;
 		if (Spgid[i].i == pgid) {
 		    Spgid[i].f = 1;
 		    *pss = PS_PRI;
@@ -683,16 +721,19 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 			return(0);
 		    break;
 		}
+		j++;
 	    }
-	    if (Selflags == SELPGID && ! *sf)
+	    if ((Selflags == SELPGID) && !*sf)
 		return(1);
 	}
 /*
- * If the listing of processes has been specified by PID, see
- * if this one is specified.
+ * If the listing of processes has been specified by PID, see if this one is
+ * included or excluded.
  */
-	if (Npid && (Selflags & SELPID)) {
-	    for (i = 0; i < Npid; i++) {
+	if (Npidi && (Selflags & SELPID)) {
+	    for (i = j = 0; (i < Npid) && (j < Npidi); i++) {
+		if (Spid[i].x)
+		    continue;
 		if (Spid[i].i == pid) {
 		    Spid[i].f = 1;
 		    *pss = PS_PRI;
@@ -701,16 +742,17 @@ is_proc_excl(pid, pgid, uid, pss, sf)
 			return(0);
 		    break;
 		}
+		j++;
 	    }
-	    if (Selflags == SELPID && ! *sf)
+	    if ((Selflags == SELPID) && !*sf)
 		return(1);
 	}
 /*
- * If the listing of processes has been specified by UID, see
- * if the owner of this process has been included.
+ * If the listing of processes has been specified by UID, see if the owner of
+* this process has been included.
  */
 	if (Nuidincl && (Selflags & SELUID)) {
-	    for (i = j = 0; i < Nuid && j < Nuidincl; i++) {
+	    for (i = j = 0; (i < Nuid) && (j < Nuidincl); i++) {
 		if (Suid[i].excl)
 		    continue;
 		if (Suid[i].uid == (uid_t)uid) {
