@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: store.c,v 1.33 2004/07/06 19:09:32 abe Exp $";
+static char *rcsid = "$Id: store.c,v 1.34 2005/05/11 13:02:46 abe Exp $";
 #endif
 
 
@@ -295,7 +295,11 @@ int NmColW;			/* NAME column width */
 char *Nmlst = (char *)NULL;	/* namelist file path */
 int NodeColW;			/* NODE column width */
 int Npgid = 0;			/* -g option count */
+int Npgidi = 0;			/* -g option inclusion count */
+int Npgidx = 0;			/* -g option exclusion count */
 int Npid = 0;			/* -p option count */
+int Npidi = 0;			/* -p option inclusion count */
+int Npidx = 0;			/* -p option exclusion count */
 int Npuns;			/* number of unselected PIDs (starts at Npid) */
 int Ntype;			/* node type (see N_* symbols) */
 int Nuid = 0;			/* -u option count */
