@@ -52,7 +52,7 @@ static char copyright[] =
  */
 
 #define SIGHANDLER_T	void		/* signal handler function type */
-#define	SOCKLEN_T	int		/* socket length type */
+#define	LT_SOCKLEN_T	int		/* socket length type */
 
 
 #if	defined(LT_DIAL_aix)
@@ -60,8 +60,8 @@ static char copyright[] =
  * AIX-specific items
  */
 
-#undef	SOCKLEN_T
-#define	SOCKLEN_T	size_t
+#undef	LT_SOCKLEN_T
+#define	LT_SOCKLEN_T	size_t
 #endif	/* defined(LT_DIAL_aix) */
 
 
@@ -70,8 +70,8 @@ static char copyright[] =
  * OpenUNIX-specific items
  */
 
-#undef	SOCKLEN_T
-#define	SOCKLEN_T	size_t
+#undef	LT_SOCKLEN_T
+#define	LT_SOCKLEN_T	size_t
 #endif	/* defined(LT_DIAL_ou) */
 
 
@@ -80,8 +80,8 @@ static char copyright[] =
  * UnixWare-specific items
  */
 
-#undef	SOCKLEN_T
-#define	SOCKLEN_T	size_t
+#undef	LT_SOCKLEN_T
+#define	LT_SOCKLEN_T	size_t
 #endif	/* defined(LT_DIAL_uw) */
 
 
@@ -140,20 +140,20 @@ pid_t MyPid = (pid_t)0;		/* PID of this process */
 char *Pn = (char *)NULL;	/* program name */
 char *PtNm[] = { "client", "server" };
 				/* program type name */
-char Ssock = -1;		/* server socket */
+int Ssock = -1;			/* server socket */
 
 
 /*
  * Local function prototypes
  */
 
-static void CleanupClnt(void);
-static void CleanupSrvr(void);
-static int ClntAlarm(int sig);
-static SIGHANDLER_T HandleClntAlarm(int sig);
-static SIGHANDLER_T HandleSrvrAlarm(int sig);
-static char *FindSock(int fn);
-static void StartClnt(struct sockaddr_in *cad);
+_PROTOTYPE(static void CleanupClnt,(void));
+_PROTOTYPE(static void CleanupSrvr,(void));
+_PROTOTYPE(static int ClntAlarm,(int sig));
+_PROTOTYPE(static SIGHANDLER_T HandleClntAlarm,(int sig));
+_PROTOTYPE(static SIGHANDLER_T HandleSrvrAlarm,(int sig));
+_PROTOTYPE(static char *FindSock,(int fn));
+_PROTOTYPE(static void StartClnt,(struct sockaddr_in *cad));
 
 
 /*
@@ -161,10 +161,9 @@ static void StartClnt(struct sockaddr_in *cad);
  */
 
 int
-main(
-    int argc,				/* argument count */
-    char *argv[]			/* arguments */
-)
+main(argc, argv)
+    int argc;				/* argument count */
+    char *argv[];			/* arguments */
 {
     struct sockaddr_in aa;		/* accept address */
     struct sockaddr_in ba;		/* bind address */
@@ -179,7 +178,7 @@ main(
     char *ipaddr;			/* IP address */
     char *pem = (char *)NULL;		/* previous error message */
     char *port;				/* port */
-    SOCKLEN_T sal;			/* socket address length */
+    LT_SOCKLEN_T sal;			/* socket address length */
     char *tcp;				/* temporary character size */
     int ti, tj, tk;			/* temporary indexes */
     int tsfd;				/* temporary socket FD */
@@ -314,7 +313,7 @@ print_errno:
 	ep = "bind";
 	goto print_errno_by_ti;
     }
-    sal = (SOCKLEN_T)sizeof(ca);
+    sal = (LT_SOCKLEN_T)sizeof(ca);
     if (getsockname(Ssock, (struct sockaddr *)&ca, &sal)) {
 	ep = "getsockname";
 	goto print_errno_by_ti;
@@ -361,11 +360,11 @@ print_errno:
 /*
  * Set a SIGALRM, then accept() the connection from the client.
  *
- * Save the client'ss socket address.
+ * Save the client's socket address.
  *
  * Replace the server's FD with the accepted one and close the original.
  */
-    sal = (SOCKLEN_T)sizeof(aa);
+    sal = (LT_SOCKLEN_T)sizeof(aa);
     (void) alarm(0);
     (void) signal(SIGALRM, HandleSrvrAlarm);
     (void) alarm(ALARMTM);
@@ -424,7 +423,7 @@ print_errno:
     }
 /*
  * Call lsof three times to find the two sockets: 1) by host name and port;
- * 2)  by IP addressand port ; and 3) by port.
+ * 2) by IP address and port; and 3) by port.
  */
     if ((cem = FindSock(LT_FBYHN)))
 	PrtMsgX(cem, Pn, CleanupSrvr, 1);
@@ -550,9 +549,8 @@ CleanupSrvr()
  */
 
 static char *
-FindSock(
-    int fn				/* function -- an LT_FBY* value */
-)
+FindSock(fn)
+    int fn;				/* function -- an LT_FBY* value */
 {
     char buf[2048];			/* temporary buffer */
     int bufl = sizeof(buf);		/* size of buf[] */
@@ -773,9 +771,8 @@ FindSock(
  */
 
 static SIGHANDLER_T
-HandleClntAlarm(
-    int sig				/* the signal (SIGALRM) */
-)
+HandleClntAlarm(sig)
+    int sig;				/* the signal (SIGALRM) */
 {
     (void) PrtMsgX("ERROR!!!  client caught an alarm signal", Pn,
 	CleanupClnt, 1);
@@ -787,9 +784,8 @@ HandleClntAlarm(
  */
 
 static SIGHANDLER_T
-HandleSrvrAlarm(
-    int sig				/* the signal (SIGALRM) */
-)
+HandleSrvrAlarm(sig)
+    int sig;				/* the signal (SIGALRM) */
 {
     (void) PrtMsgX("ERROR!!!  server caught an alarm signal.", Pn,
 	CleanupSrvr, 1);
@@ -801,9 +797,8 @@ HandleSrvrAlarm(
  */
 
 static void
-StartClnt(
-    struct sockaddr_in *cad		/* connection address */
-)
+StartClnt(cad)
+    struct sockaddr_in *cad;		/* connection address */
 {
     struct sockaddr_in ba;		/* bind address */
     int br;				/* bytes read */
@@ -841,7 +836,7 @@ client_errno:
 /*
  * Set an alarm timeout and connect to the server.
  */
-    (void) signal(SIGALRM, HandleSrvrAlarm);
+    (void) signal(SIGALRM, HandleClntAlarm);
     (void) alarm(ALARMTM);
     cr = connect(fd, (struct sockaddr *)cad, sizeof(struct sockaddr_in));
     (void) alarm(0);
