@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1998 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: usage.c,v 1.21 2004/03/10 23:44:56 abe Exp $";
+static char *rcsid = "$Id: usage.c,v 1.23 2005/05/11 13:02:46 abe Exp $";
 #endif
 
 
@@ -372,12 +372,22 @@ usage(xv, fh, version)
 
 	    (void) fprintf(stderr, " [+|-L [l]]");
 
-#if	defined(HASMOPT)
-	    (void) fprintf(stderr, " [-m m]");
-#endif	/* defined(HASMOPT) */
+#if	defined(HASMOPT) || defined(HASMNTSUP)
+	    (void) fprintf(stderr,
+# if	defined(HASMOPT)
+#  if	defined(HASMNTSUP)
+		" [+|-m [m]]"
+#  else	/* !defined(HASMNTSUP) */
+		" [-m m]"
+#  endif	/* defined(HASMNTSUP) */
+# else	/* !defined(HASMOPT) */
+		" [+m [m]]"
+# endif	/* defined(HASMOPT) */
+		);
+#endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
 	    (void) fprintf(stderr,
-	        " [+|-M] [-o [o]] [-p s]\n [+|-r [t]] [-S [t]] [-T [t]]");
+	        " [+|-M] [-o [o]]\n[-p s] [+|-r [t]] [-S [t]] [-T [t]]");
 	    (void) fprintf(stderr, " [-u s] [+|-w] [-x [fl]]");
 
 #if	defined(HASZONES)
@@ -535,13 +545,22 @@ usage(xv, fh, version)
 	    (void) fprintf(stderr,
 		"  +|-L [l] list (+) suppress (-) link counts < l (0 = all; default = 0)\n");
 
+#if	defined(HASMOPT) || defined(HASMNTSUP)
 # if	defined(HASMOPT)
 	    (void) snpf(buf, sizeof(buf), "-m m   kernel memory (%s)", KMEM);
-#else	/* !defined(HASMOPT) */
-	    (void) snpf(buf, sizeof(buf), " ");
-#endif	/* defined(HASMOPT) */
+# else	/* !defined(HASMOPT) */
+	    buf[0] = '\0';
+# endif	/* defined(HASMOPT) */
 
 	    (void) fprintf(stderr, "  %-36.36s", buf);
+
+# if	defined(HASMNTSUP)
+	    (void) fprintf(stderr, "  +m [m] use|create mount supplement\n");
+# else	/* !defined(HASMNTSUP) */
+	    (void) fprintf(stderr, "\n");
+# endif	/* defined(HASMNTSUP) */
+#endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
+
 	    (void) snpf(buf, sizeof(buf), "+|-M   portMap registration (%s)",
 
 #if	defined(HASPMAPENABLED)
@@ -551,14 +570,14 @@ usage(xv, fh, version)
 #endif	/* defined(HASPMAPENABLED) */
 
 	    );
-	    (void) fprintf(stderr, "  %s\n", buf);
+	    (void) fprintf(stderr, "  %-36.36s", buf);
 	    (void) snpf(buf, sizeof(buf), "-o o   o 0t offset digits (%d)",
 		OFFDECDIG);
-	    (void) fprintf(stderr, "  %-36.36s", buf);
-	    (void) fprintf(stderr, "  -p s   select by PID set\n");
-	    (void) snpf(buf, sizeof(buf),
-		"-S [t] t second stat timeout (%d)", TMLIMIT);
-	    (void) fprintf(stderr, "  %-36.36s", buf);
+	    (void) fprintf(stderr, "  %s\n", buf);
+	    (void) fprintf(stderr, "  %-36.36s",
+		"-p s   exclude(^)|select PIDs");
+	    (void) fprintf(stderr, "  -S [t] t second stat timeout (%d)\n",
+		TMLIMIT);
 	    (void) snpf(buf, sizeof(buf),
 		"-T %s%ss%s TCP/TPI %s%sSt%s (s) info",
 
@@ -607,8 +626,7 @@ usage(xv, fh, version)
 #endif	/* defined(HAS_AFS) && defined(HASAOPT) */
 
 	    (void) fprintf(stderr,
-		"  -g [s] select by process group ID set and print");
-	    (void) fprintf(stderr, " process group IDs\n");
+		"  -g [s] exclude(^)|select and print process group IDs\n");
 	    (void) fprintf(stderr, "  -i i   select by IPv%s address:",
 
 #if	defined(HASIPv6)
